package com.tztzf.network.cosign;

import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.network.define.tztZFErrorMessageDefine;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import static com.tztzf.network.cosign.tztZFCosignErr.tztZFCosignErrParam;

/**
 * 协同签名用户接口类
 */
public class tztZFCosignUser {
    /** 用户ID */
    private String loginUserID;
    /** pin码 */
    private String loginPin;
    /** 用户token */
    private String loginCaToken;
    /** 登录token */
    private String loginToken;
    /** 证书信息 */
    private tztZFCosignCert loginCosignCert;
    /** 加载读取错误号 errorno=1成功 */
    private tztZFCosignLoginError loginError;

    private Object context;
    /** 是否需要国密登录 */
    private boolean needGMLogin = true;

    /** AppCosigntype */
    private int appCosigntype;
    //region get/set

    public String getLoginUserID() {
        return loginUserID;
    }

    public void setLoginUserID(String loginUserID) {
        this.loginUserID = loginUserID;
    }

    public String getLoginPin() {
        return loginPin;
    }

    public void setLoginPin(String loginPin) {
        this.loginPin = loginPin;
    }

    public String getLoginCaToken() {
        return loginCaToken;
    }

    public void setLoginCaToken(String loginCaToken) {
        this.loginCaToken = loginCaToken;
    }

    public String getLoginToken() {
        return loginToken;
    }

    public void setLoginToken(String loginToken) {
        this.loginToken = loginToken;
    }

    public tztZFCosignCert getLoginCosignCert() {
        return loginCosignCert;
    }

    public void setLoginCosignCert(tztZFCosignCert loginCosignCert) {
        this.loginCosignCert = loginCosignCert;
    }

    public tztZFCosignLoginError getLoginError() {
        return loginError;
    }

    public Object getContext() {
        return context;
    }

    public void setContext(Object context) {
        this.context = context;
    }

    public boolean isNeedGMLogin() {
        return needGMLogin;
    }

    public void setNeedGMLogin(boolean needGMLogin) {
        this.needGMLogin = needGMLogin;
    }

    public int getAppCosigntype() {
        return appCosigntype;
    }

    public void setAppCosigntype(int appCosigntype) {
        this.appCosigntype = appCosigntype;
    }

    //endregion
    private tztZFCosignUser() {
    }

    /**
     * 根据userID创建
     *
     * @param context context
     * @param userID  用户ID
     */
    public tztZFCosignUser(Object context, String userID) {
        this.context = context;
        loginUserID = userID;
    }

    /**
     * 根据用户ID以及Pin码创建
     *
     * @param context context
     * @param userID  用户ID
     * @param pin     pin码
     */
    public tztZFCosignUser(Object context, String userID, String pin) {
        this.context = context;
        loginUserID = userID;
        loadCertWithPin(pin);
    }

    /**
     * 当前用户ID
     */
    public String userID() {
        return this.loginUserID;
    }

    /**
     * 通过pin码加载证书
     *
     * @param pin pin码
     * @return true=成功，还要根据loginError判断证书是否读到
     */
    public boolean loadCertWithPin(String pin) {
        if (loginUserID == null || loginUserID.length() < 1) {
            loginError = new tztZFCosignLoginError(tztZFCosignErrParam, "用户ID为空");
            return false;
        }
        if (pin == null || pin.length() < 1) {
            this.loginPin = null;
            this.loginCosignCert = null;
            String pinLable = tztZFErrorMessageDefine.getUserDefineError(tztZFErrorMessageDefine.tztZFNetWorkErrorKey_PinLable, tztZFErrorMessageDefine.tztZFPinLable);
            loginError = new tztZFCosignLoginError(tztZFCosignErrParam, String.format(Locale.getDefault(),"用户%s为空", pinLable));
            return true;
        }
        if (loginPin != null && loginPin.length() > 0 && loginCosignCert != null && pin.equals(loginPin))
            return true;

        this.loginPin = String.format(Locale.getDefault(), "%s", pin);
        List<tztZFCosignLoginError> error = new ArrayList<>();
        this.loginCosignCert = tztZFCosignCert.loadWithUserID(loginUserID, loginPin, error);
        if (error != null && error.size() > 0 && error.get(0) != null) {
            loginCosignCert = null;
            loginError = error.get(0);
        } else {
            loginError = null;
        }
        return this.loginCosignCert != null;
    }

    /**
     * 用pin码创建新证书
     *
     * @param pin pin码
     */
    public boolean newCertWithPin(String pin) {
        if (loginUserID == null || loginUserID.length() < 1)
            return false;
        if (pin == null || pin.length() < 1)
            return false;
        if (loginPin != null && loginPin.length() > 0 && loginCosignCert != null && pin.equals(this.loginPin))
            return true;
        this.loginPin = pin;
        this.loginError = null;
        this.loginCosignCert = tztZFCosignCert.initWithUserID(loginUserID, pin);
        return this.loginCosignCert != null;
    }

    /**
     * 是否有效
     */
    public boolean isValid() {
        return (loginUserID != null && loginUserID.length() > 0 && loginPin != null && loginPin.length() > 0);
    }

    /**
     * 校验PIN码
     *
     * @param pin pin码
     * @return true=成功
     */
    public boolean verifyPin(String pin) {
        if (tztZFTextUtils.isEmpty(pin))
            return false;
        if (!tztZFTextUtils.isEmpty(loginPin)) {
            return loginPin.equals(pin);
        }

        return tztZFCosignCert.loadWithUserID(loginUserID, pin, null) != null;
    }

    /**
     * 是否可协同签名
     */
    public boolean isCosigned() {
        if (appCosigntype == 0) {
            return isValid() && loginCosignCert != null && loginCosignCert.isCosigned();
        }
        return isValid();
    }

    /**
     * 本地是否存在证书
     */
    public boolean haveCosignCert() {
        if (loginCosignCert == null)
            return false;
        if (!isValid())
            return false;
        return loginCosignCert.haveCert();
    }

    /**
     * 有效期判断
     *
     * @return true=证书未过期
     */
    public boolean isValidity() {
        if (loginCosignCert == null)
            return false;
        if (appCosigntype == 0)
            return loginCosignCert.isValidity();
        return true;
    }

    /**
     * 获取证书序列号SN
     */
    public String cosignCertSN() {
        if (loginCosignCert != null && loginCosignCert.getSignInfo() != null) {
            return loginCosignCert.getSignInfo().getSerialNumber();
        }
        return "";
    }

    /**
     * 删除证书
     */
    public void removeUserCert() {
        tztZFCosignCert.removeLocalCertZZ(loginUserID);
        loginCosignCert = null;
    }

    /**
     * 获取设备唯一号
     */
    public static String getDeviceID() {
        return tztZFCosignCert.getDeviceID();
    }

    /**
     * 获取设备信息，以（,）进行分割，目前依次为设备类型，设备名称，设备系统
     */
    public static String getDeviceInfo() {
        return tztZFCosignCert.getDeviceInfo();
    }

    public String string() {
        return String.format("loginUserID=%s;loginPin=%s;loginCaToken=%s;loginToken=%s;needGMLogin=%b;appCosigntype=%s;loginError=%s;loginCosignCert=%s",
                loginUserID, loginPin, loginCaToken, loginToken, needGMLogin, appCosigntype
               , loginError == null ? "" : loginError.toString()
               , loginCosignCert == null ? "" : loginCosignCert.toString());

    }

}
