package com.tztzf.network.cosign;

import com.tztzf.tztZFPrivate;

@tztZFPrivate(name = "内部类，外层禁止调用")
public class tztZFOutputStream {
    private int offset;
    private byte[] bytes;
    private int len;
    private int lastErr;
    private String lastErrMsg;

    private tztZFOutputStream() {
    }

    public static tztZFOutputStream initWitBytes(byte[] bytes) {
        tztZFOutputStream output = new tztZFOutputStream();
        output.bytes = bytes;
        if (bytes != null) {
            output.len = bytes.length;
        }
        output.offset = 0;
        output.setError(0);
        return output;
    }

    public static tztZFOutputStream stream(byte[] bytes) {
        return tztZFOutputStream.initWitBytes(bytes);
    }

    private void setError(int error) {
        lastErr = error;
        switch (lastErr) {
            case 0:
                lastErrMsg = "";
                break;
            case 1:
                lastErrMsg = "数据已读完";
                break;
            case -1:
                lastErrMsg = "数据读取越界";
                break;
            case -2:
                lastErrMsg = "参数错误";
                break;
            case -3:
                lastErrMsg = "格式错误";
                break;
            default:
                lastErrMsg = "其他错误";
                break;
        }
    }

    /**
     * 已读完
     */
    private void readEnd() {
        offset = len;
        setError(1);
    }

    /**
     * 偏移size
     * @param size 偏移大小
     */
    private void offset(long size) {
        if (size <= 0)
            return;
        if (offset + size > len) {
            //越界
            setError(-1);
            offset = len;
        } else if (offset + size == len) {
            //读完
            readEnd();
        } else {
            //偏移size
            offset += size;
        }
    }

    /**
     * 是否读完
     * @return true=读完
     */
    private boolean isReadEnd() {
        return offset == len;
    }

    /**
     * 是否不可读
     * @return true=不可读
     */
    private boolean isReadBreak() {
        return (offset >= len);
    }

    /**
     * 获取错误
     */
    private int getErrorNo() {
        return lastErr;
    }

    /**
     * 获取错误信息
     */
    private String getErrorMsg() {
        return lastErrMsg;
    }

    /**
     * 是否还有have长度可读
     * @param have 可读长度
     * @return true=有have长度可读
     */
    private boolean isReadHaveLen(long have) {
        if (isReadBreak())
            return false;
        //可读长度判断
        return (len - offset >= have);
    }

    /**
     * 读取一个字节
     */
    private byte readChar() {
        int size = 1;
        if (isReadBreak())
            return 0;
        byte cVal = 0;
        cVal = bytes[offset + 1];
        //偏移size
        offset(size);
        return cVal;
    }

    private int readShort() {
        int size = 2;
        if (isReadBreak())
            return 0;
        if (!isReadHaveLen(size)) {
            setError(-1);
            return 0;
        }
        int nVal = getInt(bytes, offset, size);
        //偏移size
        offset(size);
        return nVal;
    }

    public int readInt() {
        int size = 4;
        if (isReadBreak())
            return 0;
        if (!isReadHaveLen(size)) {
            setError(-1);
            return 0;
        }
        int value = getInt(bytes, offset, size);
        //
        offset(size);
        return value;
    }

    private long readInt64() {
        int size = 8;
        if (isReadBreak())
            return 0;
        if (!isReadHaveLen(size)) {
            setError(-1);
            return 0;
        }
        long nVal = getLong(bytes, offset, size);
        //
        offset(size);
        return nVal;
    }

    private float readFloat() {
        int size = 4;
        if (isReadBreak())
            return 0;
        if (!isReadHaveLen(size)) {
            setError(-1);
            return 0;
        }
        float fValue = getFloat(bytes, offset);
        //
        offset(size);
        return fValue;
    }

    private double readDouble() {
        int size = 8;
        if (isReadBreak())
            return 0;
        if (!isReadHaveLen(size)) {
            setError(-1);
            return 0;
        }
        double dVal = getDouble(bytes, offset, size);
        //
        offset(size);
        return dVal;
    }


    private byte[] readBytes(long len) {
        if (len == 0)
            return null;
        if (isReadBreak())
            return null;
        if (len < 0) {
            int nLen = this.len - offset;
            byte[] data = new byte[nLen];
            System.arraycopy(bytes, offset, data, 0, nLen);
            readEnd();
            return data;
        }
        if (!isReadHaveLen(len)) {
            setError(-1);
            return null;
        }

        byte[] data = new byte[(int) len];
        System.arraycopy(bytes, offset, data, 0, (int) len);
        offset(len);
        return data;
    }

    private String readStr(int len) {
        if (len == 0)
            return "";
        if (isReadBreak())
            return "";
        if (len < 0) {
            byte[] data = readBytes(len);
            String str = new String(data);
            readEnd();
            return str;
        }
        if (!isReadHaveLen(len)) {
            setError(-1);
            return "";
        }
        byte[] data = readBytes(len);
        String str = new String(data);
        return str;
    }

    private byte[] readBytes() {
        return readBytes(-1);
    }

    private String readStr() {
        return readStr(-1);
    }

    private byte[] readLenBytesWithSize(int size) {
        if (size > 8 || size <= 0) {
            setError(-2);
            return null;
        }
        if (isReadBreak())
            return null;
        if (!isReadHaveLen(size)) {
            setError(-1);
            return null;
        }

        long len = getLong(bytes, offset, size);
        if (len < 0) {
            setError(-3);
            return null;
        }
        offset(size);
        if (!isReadHaveLen(len)) {
            setError(-1);
            return null;
        }
        return readBytes(len);
    }

    private String readLenStrWithSize(int size) {
        if (size > 8 || size <= 0) {
            setError(-2);
            return "";
        }
        if (isReadBreak())
            return "";
        if (!isReadHaveLen(size)) {
            setError(-1);
            return "";
        }
        long len = getLong(bytes, offset, size);
        if (len < 0) {
            setError(-3);
            return "";
        }
        offset(size);
        if (!isReadHaveLen(len)) {
            setError(-1);
            return "";
        }
        return readStr((int) len);
    }

    public byte[] readLenBytes() {
        return readLenBytesWithSize(4);
    }

    public String readLenStr() {
        return readLenStrWithSize(4);
    }

    private static int getInt(byte[] data, int start, int length) {
        int temp = 0;
        for (int i = 0; i < length; i++) {
            temp |= (data[i + start] & 0xff) << i * 8;
        }
        return temp;
    }

    private static long getLong(byte[] data, int start, int length) {
        long temp = 0;
        for (int i = 0; i < length; i++) {
            temp |= (data[i + start] & 0xff) << (i) * 8;
        }
        return temp;
    }

    /**
     * byte数组转换浮点型
     * @param in byte数组
     * @param start 开始位置
     * @return 转换后的浮点型数值
     */
    private static long getFloat(byte[] in, int start) {
        long a, b, c, d;
        long temp, temp2;
        a = in[start]; // 237/211
        b = in[start + 1];// 213//143
        c = in[start + 2]; // 135//167
        d = in[start + 3];// 79//
        temp = 0;
        temp2 = d >> 7;
        // MS为无符号byte,java是有符号byte需加下面转换
        a = a < 0 ? a + 256 : a;
        b = b < 0 ? b + 256 : b;
        c = c < 0 ? c + 256 : c;
        d = d <= 0 ? d + 256 : d;

        if (temp2 < 0)
            d = d ^ 128;

        long size = (d << 1) + (c >> 7) - 127;
        c = c | 128;
        if (size <= 7) {
            temp = c >> (7 - size);
        } else if ((size > 7) && (size <= 15)) {
            temp = (b >> (15 - size)) + (c << (size - 7));
        } else if ((size > 15) && (size <= 23)) {
            temp = (a >> (23 - size)) + (b << (size - 15)) + (c << (size - 7));
        } else if (size > 23) {
            temp = (a << (size - 23)) + (b << (size - 15)) + (c << (size - 7));
        }

        temp = temp2 < 0 ? temp * temp2 : temp;
        return temp;
    }

    private static double getDouble(byte[] bytes, int start, int len) {
        long ff = 0xFF;
        long bitLayoutLongValue = 0;
        for (int i = 0; i < len; i++) {
            bitLayoutLongValue |= (bytes[i + start] & ff) << (bytes.length - i - 1) * 8;
        }
        return Double.longBitsToDouble(bitLayoutLongValue);
    }
}
