package com.tztzf.recordprocesslog;

import com.tztzf.baseos.tztZFDoneFile;
import com.tztzf.baseos.tztZFTextUtils;
import com.tztzf.log.tztZFLog;

import org.json.JSONArray;
import org.json.JSONObject;

import java.lang.ref.WeakReference;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Locale;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.RejectedExecutionHandler;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * ****************************************************************
 * 文件名称 : tztZFRecordLogTools.java
 * 作   者 : Created by 王路子
 * 创建时间 : 2023-06-28 13:20
 * 文件描述 : 通讯层日志记录工具
 * 注意事项 :
 * 版权声明 : Copyright (C) 2020-2030 杭州中焯信息技术股份有限公司
 * 修改历史 : 2023-06-28 1.00 初始版本
 * ****************************************************************
 */
public class tztZFRecordProcessLogTools {

    /** 是否记录日志 */
    private WeakReference<Object> context;
    private boolean bSwitch;
    private String fileName = "tztZFNetWorkProcessLog.txt";
    private WeakReference<tztZFRecordProcessLogCallback> callback;
    private ThreadPoolExecutor m_pThreadResp = null;
    private static tztZFRecordProcessLogTools ins;

    public static tztZFRecordProcessLogTools getIns() {
        if (ins == null) {
            ins = new tztZFRecordProcessLogTools();
        }
        return ins;
    }

    public tztZFRecordProcessLogTools() {
        //文件名做MD5加密
        if (m_pThreadResp == null) {
            m_pThreadResp = new ThreadPoolExecutor(
                    1,      //核心线程数设置成1
                    1,  //线程池最大线程数为1
                    10, TimeUnit.SECONDS, //超过核心线程数量的线程idle 10秒之后会退出
                    new ArrayBlockingQueue<Runnable>(50), //队列长度为50
                    new tztRecordDiscardOldestPolicy() //任务被拒绝以后的处理类
            );
        }
    }

    /**
     * 是否记录
     * @return true=记录；false=不记录
     */
    public static boolean isRecord() {
        return tztZFRecordProcessLogTools.getIns().bSwitch;
    }

    /**
     * 设置是否记录通讯流程日志
     * @param context Context上下文
     * @param bSwitch 是否记录 true=记录；false=不记录
     */
    public static void setRecordSwitch(Object context, boolean bSwitch, tztZFRecordProcessLogCallback callback) {
        tztZFRecordProcessLogTools.getIns().context = new WeakReference<>(context);
        tztZFRecordProcessLogTools.getIns().bSwitch = bSwitch;
        tztZFRecordProcessLogTools.getIns().callback = new WeakReference<>(callback);
//        tztZFRecorderLogDBTools.setContext(tztZFRecordProcessLogTools.getIns().context.get());
    }

    /**
     * 记录流程日志
     * @param linkKey 通道key
     * @param recordEventType 事件类型
     * @param reocordLogType 记录日志类型
     * @param params 事件信息
     * @param strStackTrace 运行栈信息
     */
    public static synchronized void recordLog(String linkKey, @tztZFRecordEventType int recordEventType, @tztZFReocordLogType int reocordLogType, ArrayList<Object> params, String strStackTrace) {
        tztZFRecordProcessLogTools.getIns().recordLogEx(linkKey, recordEventType, reocordLogType, params, strStackTrace);
    }

    private void recordLogEx(final String linkKey, final @tztZFRecordEventType int recordEventType, final @tztZFReocordLogType int reocordLogType, final ArrayList<Object> params,
                             String strStackTrace) {
        if (!bSwitch) {
            return;
        }

        //运行栈信息
        if (tztZFTextUtils.isEmpty(strStackTrace)) {
            strStackTrace = tztZFLog.getStackTraceString(new Exception());
        }
        final String stackTrace = strStackTrace;
        m_pThreadResp.execute(new Runnable() {
            @Override
            public void run() {
                try {
                    String[] events = tztZFRecordEventType.tztZFRecordEventTypeTips.tips(recordEventType);
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("event_LinkKey", linkKey);
                    jsonObject.put("event_ID", events[0]);
                    jsonObject.put("event_Name", events[1]);
                    jsonObject.put("event_Time", convertTimeMillisToString(System.currentTimeMillis(), "yyyyMMddHHmmssSSS"));
                    if (params != null && params.size() > 0) {
                        JSONArray jsonArray = new JSONArray();
                        for (int i = 0; i < params.size(); i++) {
                            jsonArray.put(params.get(i));
                        }
                        jsonObject.put("event_Params", jsonArray.toString());
                    }
                    jsonObject.put("event_StackTrace", stackTrace);
                    String log = jsonObject.toString() + "\r\n\n";
                    //回调上层
                    if (callback != null && callback.get() != null) {
                        callback.get().callback(log, reocordLogType);
                    } else if (context != null && context.get() != null) {
                        //记录数据库
//                        tztZFRecorderLogDBTools.insert(log);
                        tztZFDoneFile.writeFileDataWithAppend(context.get(), fileName, log);
                    }
                } catch (Exception e) {

                }
            }
        });
    }


    public static ArrayList<Object> covertParams(Object... args) {
        if (args == null) {
            return new ArrayList<>();
        }
        ArrayList<Object> arrayList = new ArrayList<>();
        for (int i = 0; i < args.length; i++) {
            if (args[i] == null) {
                continue;
            }
            arrayList.add(args[i]);
        }
        return arrayList;
    }

    public static String stringFormat(String format, Object... args) {
        try {
            return String.format(Locale.CHINA, format, args);
        } catch (Exception e) {

        }
        return format;
    }

    private String convertTimeMillisToString(long timeMillis, String format) {
        return convertDateToString(new Date(timeMillis), format);
    }

    /**
     * 日期转换成字符串
     *
     * @param date   日期
     * @param format 字符串格式
     * @return 日期字符串
     */
    private String convertDateToString(Date date, String format) {
        if (date == null || format == null) {
            return null;
        }
        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat(format, Locale.getDefault());
            return dateFormat.format(date);
        } catch (Exception e) {
        }
        return null;
    }

    class tztRecordDiscardOldestPolicy implements RejectedExecutionHandler {
        @Override
        public void rejectedExecution(Runnable r, ThreadPoolExecutor executor) {
            // 处理任务被拒绝的情况，记录日志
            //弹出最先加入的任务，空出一个位置，然后再次执行execute方法把任务加入队列。
            if (!executor.isShutdown()) {
                Runnable p = executor.getQueue().poll();
                if (p != null) {
//                    tztlog(, "ThreadResp", "%s", p.toString());
                }
                executor.execute(r);
            }
        }
    }

}